/**
 * EasyBeans
 * Copyright (C) 2006-2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: AbstractInjectionHandler.java 4124 2008-10-06 13:57:27Z sauthieg $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.annotation.processor.handler;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.ow2.util.annotation.processor.IAnnotationHandler;
import org.ow2.util.annotation.processor.ProcessorException;

/**
 * An {@link AbstractInjectionHandler} is a simple {@link IAnnotationHandler}
 * that provides facility methods to perform field and method injection.
 *
 * @author Guillaume Sauthier
 */
public abstract class AbstractInjectionHandler extends AbstractAnnotationHandler {

    /**
     * Inject a given value in the given object, using the 'setter'
     * method provided as parameter.
     *
     * @param method setter method
     * @param target instance to be injected
     * @param value  value to be injected
     * @throws ProcessorException
     */
    protected void doInjectMethod(final Method method,
                                  final Object target,
                                  final Object value) throws ProcessorException {
        boolean accessibility = method.isAccessible();

        try {
            method.setAccessible(true);
            method.invoke(target, value);
        } catch (IllegalArgumentException e) {
            throw new ProcessorException("Cannot inject '" + method + "'", e);
        } catch (IllegalAccessException e) {
            throw new ProcessorException("Cannot inject '" + method + "'", e);
        } catch (InvocationTargetException e) {
            throw new ProcessorException("Cannot inject '" + method + "'", e);
        } finally {
            method.setAccessible(accessibility);
        }
    }

    /**
     * Inject a given value in the given object, using the field provided
     * as parameter.
     *
     * @param field  injection field
     * @param target instance to be injected
     * @param value  value to be injected
     * @throws ProcessorException
     */
    protected void doInjectField(final Field field,
                                 final Object target,
                                 final Object value) throws ProcessorException {
        boolean accessibility = field.isAccessible();

        try {
            field.setAccessible(true);
            field.set(target, value);
        } catch (IllegalArgumentException e) {
            throw new ProcessorException("Cannot inject '" + field + "'", e);
        } catch (IllegalAccessException e) {
            throw new ProcessorException("Cannot inject '" + field + "'", e);
        } finally {
            field.setAccessible(accessibility);
		}
	}

}
